@@ -60,6 +60,17 @@ $usertable_prefix = 'phplist_user_';
 // install and initialise phpList.
 $pageroot = '/lists';
 
+
+// there are places in phpList where files need to be fetched from the internet.
+// for example translations, or lists of valid domains
+// also the phpList updater needs access to the phpList servers
+// if your system is behind a proxy server, provide the details here
+
+#define('HTTP_PROXY_HOST', 'proxy.mydomain.com');
+// ex 80. Usually 80,8080 or 3128
+#1define('HTTP_PROXY_PORT', '80');
+
+
 /*
 
 =========================================================================
@@ -126,6 +137,18 @@ $bounce_unsubscribe_threshold = 5;
 // Set to 0 to received by mail bounce deletions in the advanced bounce processing report
 define('REPORT_DELETED_BOUNCES', 0);
 
+// The name of the POP3 mailbox
+// Multiple mailboxes can be specified separated by comma
+$bounce_mailbox_name = 'INBOX';
+
+// The maximum number of bounces to retrieve from the mailbox
+// This might need to be reduced if the processing times-out
+$bounce_mailbox_maximum = 100000;
+
+// When applying bounce rules the number of bounces to process in each batch
+// This might need to be reduced when there are a large number of bounces to process
+$bounce_rules_batch_size = 500;
+
 /*
 
 =========================================================================
@@ -177,9 +200,13 @@ define('HASH_ALGO', 'sha256');
 // to 1 for this to have an effect
 define('UNSUBSCRIBE_REQUIRES_PASSWORD', 0);
 
-// if a user should immediately be unsubscribed, when using their personal URL, instead of
-// the default way, which will ask them for a reason, set this to 1
-define('UNSUBSCRIBE_JUMPOFF', 0);
+// Immediately unsubscribe a subscriber when using their personal URL.
+// To display a confirmation page asking them for a reason set this to 0.
+// Be aware that setting to 1 might lead to unauthorised unsubscriptions due to a receiving mail server
+// "following" links in an email. If that happens then set this to 0.
+// Also, see the file README.robots in the phplist distribution doc directory for another way to try to
+// stop unauthorised unsubscriptions.
+define('UNSUBSCRIBE_JUMPOFF', 1);
 
 // To not send confirmation of unsubscription , instead of
 // the default way, which will send it, set this to false
@@ -604,7 +631,7 @@ define('PHPMAILERHOST', '');
 
 //# SMTP debugging
 // Enable debugging output by phpmailer when sending test emails
-// See https://phpmailer.github.io/PHPMailer/classes/PHPMailer.PHPMailer.PHPMailer.html#property_SMTPDebug
+// See https://phpmailer.github.io/PHPMailer/classes/PHPMailer-PHPMailer-PHPMailer.html#property_SMTPDebug
 // define('PHPMAILER_SMTP_DEBUG', 0);
 
 //# Smtp Timeout
@@ -672,6 +699,9 @@ define('EMBEDEXTERNALIMAGES',false);
 // instead of trying to create it by parsing the HTML version into plain text
 define('USE_MANUAL_TEXT_PART', 0);
 
+// Message preview tries to show a small preview of how your campaign will look in email applications
+define('USE_MESSAGE_PREVIEW',true);
+
 // set this to 1 to allow adding attachments to the mails
 // caution, message may become very large. it is generally more
 // acceptable to send a URL for download to users
@@ -875,6 +905,22 @@ define('ALLOW_UPDATER', true);
 // adds constant to email headers
 define('GOOGLE_SENDERID', '');
 
-// For ajax based signup forms (https://discuss.phplist.org/t/solved-ajax-subscribe-api/974) the access-control-allow-origin header has to be set properly.
+// For ajax based signup forms (https://discuss.phplist.org/t/solved-ajax-subscribe-api/974) the access-control-allow-origin header
+// has to be set properly.
 // Add the addresses of the websites you want to allow to perform ajax requests to PHPList.
-define('ACCESS_CONTROL_ALLOW_ORIGINS', ['https://example.com','https://example.org']);
\ No newline at end of file
+define('ACCESS_CONTROL_ALLOW_ORIGINS', ['https://example.com','https://example.org']);
+
+// when using Mysql, what engine should we use
+// there is no need to set this, when you have a valid engine defined in your Mysql server settings
+// If you set this, the next time upgrade is run on commandline, the system will attempt to change all tables to this engine.
+// this is useful for old systems that have been created
+// for valid engines, check your server documentation (MariaDB or Mysql)
+// Warning, setting this value incorrectly will cause your database creation to break
+// the current value is a sensible one to use. It is advised not to change this.
+// if you are on a shared hosting environment, make sure you have the database permissions to update the
+// information.schema tables.
+$mysql_database_engine = 'InnoDB';
+
+// The subscriber totals for each list on the lists page are calculated by sending a sequence of asynchronous requests.
+// You can include a delay between requests using this setting. The value is in milliseconds.
+define('ASYNC_REQUEST_INTERVAL', 0);
